/*
 * Copyright (c) 2007, 2008, 2009, 2010, 2011, 2012 David Berkman
 * 
 * This file is part of the SmallMind Code Project.
 * 
 * The SmallMind Code Project is free software, you can redistribute
 * it and/or modify it under the terms of GNU Affero General Public
 * License as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 * 
 * The SmallMind Code Project is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the the GNU Affero General Public
 * License, along with The SmallMind Code Project. If not, see
 * <http://www.gnu.org/licenses/>.
 * 
 * Additional permission under the GNU Affero GPL version 3 section 7
 * ------------------------------------------------------------------
 * If you modify this Program, or any covered work, by linking or
 * combining it with other code, such other code is not for that reason
 * alone subject to any of the requirements of the GNU Affero GPL
 * version 3.
 */
package org.smallmind.swing.catalog;

import org.smallmind.nutsnbolts.util.DirectionalComparator;

public class MultiListSelectionRange<T extends Comparable<T>> {

  public static enum Containment {

    SURROUNDS, HEAD, TAIL, WITHIN, OUT
  }

  ;

  private MultiListSelection<T> firstSelection;
  private MultiListSelection<T> lastSelection;
  private DirectionalComparator.Direction direction;

  public MultiListSelectionRange (MultiListSelection<T> selection0, MultiListSelection<T> selection1, DirectionalComparator.Direction direction) {

    int comparison;

    this.direction = direction;

    comparison = selection0.compareTo(selection1, direction);

    if (comparison <= 0) {
      firstSelection = selection0;
      lastSelection = selection1;
    }
    else {
      firstSelection = selection1;
      lastSelection = selection0;
    }
  }

  public MultiListSelection<T> getFirstSelection () {

    return firstSelection;
  }

  public MultiListSelection<T> getLastSelection () {

    return lastSelection;
  }

  public Containment getContainment (T key) {

    int firstComparison;
    int lastComparison;

    firstComparison = firstSelection.getComparable().compareTo(key);
    lastComparison = lastSelection.getComparable().compareTo(key);

    if (direction.equals(DirectionalComparator.Direction.ASCENDING)) {
      firstComparison *= -1;
      lastComparison *= -1;
    }

    if ((firstComparison == 0) && (lastComparison == 0)) {
      return Containment.SURROUNDS;
    }
    else if (firstComparison == 0) {
      return Containment.HEAD;
    }
    else if (lastComparison == 0) {
      return Containment.TAIL;
    }
    else if ((firstComparison < 0) && (lastComparison > 0)) {
      return Containment.WITHIN;
    }
    else {
      return Containment.OUT;
    }
  }

}
